/**
 * Copyright (c) 2025 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvSignalVar.h"
#include "HvSignalLine.h"
#include "HvMath.h"
#include "HvControlVar.h"
#include "HvControlBinop.h"
#include "HvControlCast.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_p8M2iQ41_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_MFASHTBO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_pd4UsX49_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_TsvrW8Tw_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_MWyQ8GJ6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_OxRzhKX6_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_PHK9VLZt_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_hA5pxPMq_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_QW3yIFKS_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_tbnH8FZd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_84TxNgmD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_ad8V8FDd_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_7y1olq6J_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_pVWAgDfv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_JZsj6eyk_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_5mTzy8fQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_roJJ8wCO_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_5QVNwNAf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_IDSE2fQn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_CLftTQTv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_M8XjOPVA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_bkR38i90_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_OqLoFzSP_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_p4pLHBUv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_rBWNBj7X_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_PLixooT9_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_qylnXtso;
  SignalLine sLine_2uvVn83F;
  SignalLine sLine_DBkwgChc;
  ControlBinop cBinop_p8M2iQ41;
  ControlBinop cBinop_MFASHTBO;
  ControlBinop cBinop_pd4UsX49;
  ControlBinop cBinop_TsvrW8Tw;
  ControlBinop cBinop_PHK9VLZt;
  ControlBinop cBinop_QW3yIFKS;
  ControlBinop cBinop_tbnH8FZd;
  ControlBinop cBinop_84TxNgmD;
  ControlVar cVar_ad8V8FDd;
  ControlVar cVar_7y1olq6J;
  SignalVarf sVarf_jwSIYGjx;
  SignalVarf sVarf_Mtb6ybNm;
  SignalVarf sVarf_ZqJ676NY;
  SignalVarf sVarf_9wAU2DZa;
  ControlBinop cBinop_5QVNwNAf;
  ControlBinop cBinop_bkR38i90;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
